package school_project.Parsers;

import org.junit.jupiter.api.*;
import school_project.Map;
import school_project.Piece;

import java.io.File;
import java.io.IOException;

import static org.junit.jupiter.api.Assertions.*;

@TestMethodOrder(value = MethodOrderer.OrderAnnotation.class)
class FileParserFactoryTest {

    static Map generateMapTest(){
        boolean[][] map_shape = {
                {true, true, true, true},
                {true, false, true, true},
                {true, true, false, true},
                {true, true, false, false},
        };
        boolean[][] piece1_shape = {
                {true, true},
                {true, false},
                {true, true},
                {true, true},
        };
        boolean[][] piece2_shape = {
                {true, true},
        };
        boolean[][] piece3_shape = {
                {true, true},
                {false, true},
        };
        Piece[] pieces = { new Piece(piece1_shape), new Piece(piece2_shape), new Piece(piece3_shape) };
        Map map = new Map(map_shape);
        map.addPiece(pieces);
        return map;
    }

    @Test
    void saveFileFromMap_Binary() throws IOException {
        Map map = generateMapTest();
        FileParserFactory.saveFileFromMap(new File("TestBinaryLevel.level"), map);
    }

    @Test
    void saveFileFromMap_save_Binary() throws IOException {
        Map map = generateMapTest();
        FileParserFactory.saveFileFromMap(new File("TestBinarySave.slevel"), map);
    }
    @Test
    void saveFileFromMap_Serialized() throws IOException {
        Map map = generateMapTest();
        FileParserFactory.saveFileFromMap(new File("TestSerializedLevel.serialized"), map);
    }

    @Test
    void saveFileFromMap_save_Serialized() throws IOException{
        Map map = generateMapTest();
        FileParserFactory.saveFileFromMap(new File("TestSerializedSave.sserialized"), map);
    }

    @Test
    void loadMapFromFile_Binary() throws IOException {
        Map map = generateMapTest();
        Map testMap = FileParserFactory.loadMapFromFile(new File("src/test/resources/saveFileExamples/TestBinaryLevel.level"));
        assertArrayEquals(map.getCleanedMap().getShape(), testMap.getShape());
        assertTrue(map.getCleanedMap().getPieces().equals(testMap.getPieces()));
    }

    @Test
    void loadMapFromFile_saved_Binary() throws IOException {
        Map map = generateMapTest();
        Map testMap = FileParserFactory.loadMapFromFile(new File("src/test/resources/saveFileExamples/TestBinarySave.slevel"));
        assertArrayEquals(map.getShape(), testMap.getShape());
        assert(map.getPieces().equals(testMap.getPieces()));
    }
    @Test
    void loadMapFromFile_Serialized() throws IOException {
        Map map = generateMapTest();
        Map testMap = FileParserFactory.loadMapFromFile(new File("src/test/resources/saveFileExamples/TestSerializedLevel.serialized"));
        assertArrayEquals(map.getCleanedMap().getShape(), testMap.getShape());
        assert(map.getCleanedMap().getPieces().equals(testMap.getPieces()));
    }

    @Test
    void loadMapFromFile_saved_Serialized() throws IOException {
        Map map = generateMapTest();
        Map testMap = FileParserFactory.loadMapFromFile(new File("src/test/resources/saveFileExamples/TestSerializedSave.sserialized"));
        assertArrayEquals(map.getShape(), testMap.getShape());
        assertTrue(map.getPieces().equals(testMap.getPieces()));
    }
}